; Logger - Message Logging Library
; By Jocelyn 'GoSsE' Perreault, http://www.nuloen.com/
; Last Updated: October 29 2003
; Version: 1.1.1
; This source code is copyrighted (c) to Jocelyn Perreault.
; It is FreeWare. You have no limitiations in regard to 
; applications/games that you release/create with it, either
; they be freeware or commercial. However, you cannot release
; this source code under another name or release a similar
; library with it. It cannot be redistributed. The code may be 
; modified to be tweaked but you cannot distribute it. If you
; made a modification that you think the community would
; benifit, please let me know.
; If you make tons of money with a commercial project,
; a free copy wouldn't hurt ;)
; I would love to have an e-mail and a place in the credits
; of your projects, but it is up to you.
; Thanks,
;                 Jocelyn "GoSsE Korupted" Perreault
;                  gosse@nuloen.com
;                  Nuclear Loaded Entertainment
;                  http://www.nuloen.com/

; ==============================================================
; Global Variables
; ==============================================================
Global L_MsgLogs.L_MsgLog
Global L_MsgColors.L_MsgColor
Global L_Messages.L_Message
Global L_Spacer$ = "*"

; ==============================================================
; Types
; ==============================================================

; Message Log
Type L_MsgLog
	Field ID						; Log ID
	Field filMsg					; File Handle of the log (if any)
	Field saveTime					; Save time or not?
	Field writeOnClose				; If it saves as it goes or on close (if true, *must* call CloseLog to save)
	Field defaultcolorID	 		; Default Color's ID
	Field offSet					; Offset to display the log
End Type

; ---------------------------

; Message Colors
Type L_MsgColor
	Field ID						; Color ID
	Field imgChars[255]				; Each image for ASCII characters
End Type

; ---------------------------

; Messages
Type L_Message
	Field LogID						; The log's ID
	Field Msg$						; The Message itself
	Field Time$						; The time at with the Message was logged
	Field clrID						; The ID of the color
End Type

; ---------------------------

; Message Lines
Type L_MsgLine
	Field Msg$
End Type

; ==============================================================
; Logger Functions
; ==============================================================

; CreateColor - Creates a color
Function L_CreateColor(r, g, b, buffer = -1)
	Local x
	Local y
	Local imgTemp
	Local i
	Local sw
	Local sh
	Local ID
	; Assign an ID
	For L_MsgColors.L_MsgColor = Each L_MsgColor
		If ID = L_MsgColors\ID Then
			ID = ID + 1
		End If
	Next
	L_MsgColors.L_MsgColor = New L_MsgColor
	L_MsgColors\ID = ID
	; Creates a temporary buffer
	imgTemp = CreateImage(640, 480)
	SetBuffer ImageBuffer(imgTemp)
	x = 5
	y = 5
	; Creates all 256 ASCII Table Characters images
	For i = 0 To 255
		sw = StringWidth(Chr(i))
		sh = StringHeight(Chr(i))
		Color 255, 0, 255
		Rect x, y, sw+2, sh+2
		Color R, G, B
		L_OutlineText x+1, y+1, Chr(i), False, False
		L_MsgColors\imgChars[i] = CreateImage(sw + 2, sh + 2)
		GrabImage L_MsgColors\imgChars[i], x, y
		MaskImage L_MsgColors\imgChars[i], 255, 0, 255
		x = x + sw + 3
		If x > 600 Then
			x = 5
			y = y + 20
		End If
	Next
	FreeImage imgTemp
	; Set the buffer back
	If buffer = -1 Then
		SetBuffer BackBuffer()
	Else
		SetBuffer buffer
	End If
	; Returns the color ID
	Return ID
End Function

; ---------------------------

; CreateLog - To create a new log and assign a file and return ID
Function L_CreateLog(defaultcolor, file$ = "", append = False, savetime = False, writeonclose = False)
	; Local Vars
	Local ID
	; Find a new ID
	ID = 0
	For L_MsgLogs.L_MsgLog = Each L_MsgLog
		If L_MsgLogs\ID = ID Then
			ID = ID + 1
		End If
	Next
	L_MsgLogs.L_MsgLog = New L_MsgLog
	L_MsgLogs\ID = ID
	L_MsgLogs\defaultcolorID = defaultcolor
	If file$ <> "" Then
		; If in append mode, open the file, and if it doesnt exist, creates it
		If append Then
			L_MsgLogs\filMsg = OpenFile(file$)
			If L_MsgLogs\filMsg Then
				SeekFile L_MsgLogs\filMsg, FileSize(file$)
			End If		
		End If
		If Not(L_MsgLogs\filMsg) Then
			L_MsgLogs\filMsg = WriteFile(file$)
		End If
		; Save parameters
		L_MsgLogs\SaveTime = savetime
		L_MsgLogs\WriteOnClose = writeonclose
	End If
	Return ID
End Function

; ---------------------------

; SetSpacer - To change the spacer character for date/Message separation
Function L_SetSpacer(newSpacer$)
	L_Spacer$ = newSpacer$
End Function

; ---------------------------

; AddMessage - To add a new L_Message to a log
Function L_AddMessage(LogID, Msg$, colorID = -1)
	; Go to the proper Log
	For L_MsgLogs.L_MsgLog = Each L_MsgLog
		If L_MsgLogs\ID = LogID Then
			Exit
		End If
	Next
	; Creates a new Message
	L_Messages.L_Message = New L_Message
	; Assign properties
	L_Messages\LogID = LogID
	L_Messages\Msg$ = Msg$
	If L_MsgLogs\SaveTime Then
		L_Messages\Time$ = CurrentDate() + " " + CurrentTime()
	End If
	If colorID = -1 Then
		L_Messages\clrID = L_MsgLogs\DefaultColorID
	Else
		L_Messages\clrID = colorID
	End If
	; Save to file if not on close
	If (Not L_MsgLogs\WriteOnClose) And (L_MsgLogs\filMsg > 0) Then
		L_WriteMsg()
	End If
End Function

; ---------------------------

; DisplayMessages - To display L_Messages from a log, within a viewport
Function L_DisplayMessages(LogID, X, Y, W, H, OffSet = 0, Display = True, Counter = False)
	; Local Vars
	Local lineHeight
	Local Over
	Local Finished
	Local Msg$
	Local j
	Local k
	Local Count
	Local CY
	; Goes through all L_Messages *backwards*
	L_Messages.L_Message = Last L_Message
	CY = Y + H + 2
	While L_Messages <> Null And (Not Over)
		If L_Messages\LogID = LogID Then
			; Get font height for that line
			lineHeight = L_Height(L_Messages\clrID, "W") + 1 - 5
			; Performs multi-lining for the viewport
			Finished = False
			j = 1
			; Constructs the lines
			While Not Finished
				Msg$ = ""
				; Constructs the current line
				While L_Width(L_Messages\clrID, Msg$) <= W And Mid(L_Messages\Msg$, j, 1) <> ""
					Msg$ = Msg$ + Mid(L_Messages\Msg$, j, 1)
					j = j + 1
				Wend
				; Removes half-words
				If L_Width(L_Messages\clrID, Msg$) >= W Then
					k = Len(Msg$)
					If k > 1 Then
						While Mid(Msg$, k, 1) <> " " And k > 1
							k = k - 1
							j = j - 1
						Wend
						If k > 1 Then
							Msg$ = Left(Msg$, k - 1)
						Else
							; If the word is too large for the line, cut it before last vowel -1
							For k = k To Len(Msg$)
								j = j + 1
							Next
							While L_Width(L_Messages\clrID, Msg$) > W
								k = k - 1
								j = j - 1
								Msg$ = Left(Msg$, k)
							Wend
							If k > 1 Then
								k = k - 1
								j = j - 1
								Msg$ = Left(msg$, k)
								While L_IsVowel(Mid(Msg$, k, 1)) And k > 1
									k = k - 1
									j = j - 1
									Msg$ = Left(msg$, k)
								Wend
								If k > 1 Then
									k = k - 1
									j = j - 1
									Msg$ = Left(msg$, k) + "-"
								End If
							End If
						End If
					End If
				End If
				If Msg$ = "" Then
					Finished = True
				Else
					L_MsgLines.L_MsgLine = New L_MsgLine
					L_MsgLines\msg$ = msg$
				End If
			Wend
			; Display the lines *backwards*
			L_MsgLines.L_MsgLine = Last L_MsgLine
			While L_MsgLines <> Null
				; Count the lines
				Count = Count + 1
				; Ensure it's within the offset
				If Count > offSet Then
					; Substract font-height
					CY = CY - lineHeight
					; Don't display if outside of viewports
					If CY >= Y - 2 And CY + lineHeight <= Y + H + 2 Then
						If Display Then
							L_Text(L_Messages\clrID, X, CY, L_MsgLines\Msg$)
						End If
					Else
						If (Not Counter) Then
							Count = Count - 1
						End If
					End If
				End If
				Delete L_MsgLines
				L_MsgLines.L_MsgLine = Last L_MsgLine
			Wend
			If CY < Y Then
				If Not Counter
					Over = True
				End If
			End If
		End If
		L_Messages = Before L_Messages
	Wend
	Return Count
End Function

; ---------------------------

; CloseLog - To close a log (and its file if any)
Function L_CloseLog(ID)
	; Find the log
	For L_MsgLogs.L_MsgLog = Each L_MsgLog
		If L_MsgLogs\ID = ID Then	
			If L_MsgLogs\filMsg Then
				; If write on close, writes all msgs
				If L_MsgLogs\WriteOnClose Then
					For L_Messages.L_Message = Each L_Message
						L_WriteMsg()
					Next
				End If
				; Close the file
				CloseFile L_MsgLogs\filMsg
				; Clears the log
				L_ClearLog(ID)
				; Deletes the log
				Delete L_MsgLogs
				Exit
			End If
		End If
	Next
End Function

; ---------------------------

; SaveLog - To save a log's file
Function L_SaveLog(ID)
	; Find the log
	For L_MsgLogs.L_MsgLog = Each L_MsgLog
		If L_MsgLogs\ID = ID Then	
			If L_MsgLogs\filMsg Then
				; If write on close, writes all msgs
				If L_MsgLogs\WriteOnClose Then
					For L_Messages.L_Message = Each L_Message
						L_WriteMsg()
					Next
				End If
				Exit
			End If
		End If
	Next
End Function

; ---------------------------

; CountLines - To count the total number of lines
Function L_CountLines(LogID, W)
	Return L_DisplayMessages(LogID, 0, 0, W, 0, 0, False, True)
End Function

; ---------------------------

; Clear Log - To clear a log, with how many messages you want to keep
Function L_ClearLog(LogID, MessagesToKeep = 0)
	Local i
	L_Messages = Last L_Message
	While L_Messages <> Null
		i = i + 1
		If i > MessagesToKeep Then
			Delete L_Messages
		End If
		L_Messages = Before L_Messages
	Wend
End Function

; ---------------------------

; LoadLog - To load a file into an existing log
Function L_LoadLog(LogID, file$, writeit = False)
	; Local Vars
	Local filLog
	Local ReadL$
	Local PosF
	Local timeS$
	Local msg$
	; Find the log
	For L_MsgLogs.L_MsgLog = Each L_MsgLog
		If L_MsgLogs\ID = LogID Then	
			; Opens the file
			filLog = ReadFile(file$)
				While Not Eof(filLog)
					ReadL$ = ReadLine(filLog)
					PosF = Instr(ReadL$, L_Spacer$)
					If PosF = 0 Then
						; No settings are saved
					Else
						timeS$ = Left(ReadL$, PosF - 1)
					End If
					; L_Message itself
					msg$ = Mid(ReadL$, PosL + 1)
					; Add this L_Message to the log
					L_Messages.L_Message = New L_Message
					; Assign properties
					L_Messages\LogID = LogID
					L_Messages\Msg$ = Msg$
					If L_MsgLogs\SaveTime Then
						If timeS$ = "" Then
							L_Messages\Time$ = CurrentDate() + " " + CurrentTime()
						Else
							L_Messages\Time$ = timeS$
						End If						
					End If
					; Assign default color
					L_Messages\clrID = L_MsgLogs\DefaultColorID
					If WriteIt Then
						L_WriteMsg()
					End If
				Wend
			CloseFile filLog
		End If
	Next
End Function

; ==============================================================
; Misc. Functions (called internally, not by the user)
; ==============================================================
Function L_WriteMsg()
	; Local Vars
	Local Msg$
	Msg$ = ""
	; Save time if need be
	If L_MsgLogs\SaveTime Then
		Msg$ = Msg$ + L_Messages\Time$ + L_Spacer$
	End If
	; Write to file
	WriteLine L_MsgLogs\filMsg, Msg$ + L_Messages\Msg$
End Function
; ---------------------------
Function L_Height(fntID, txt$)
	Local high
	Local i
	Local test
	Local ascCode
	For L_MsgColors.L_MsgColor = Each L_MsgColor
		If fntid = L_MsgColors\ID Then
			Exit
		End If
	Next
	For i = 1 To Len(txt$)
		ascCode = Asc(Mid(txt$, i, 1))
		test = ImageHeight(L_MsgColors\imgChars[ascCode])
		If test > high Then
			high = test
		End If
	Next
	Return high
End Function
; ---------------------------
Function L_Width(fntID, txt$, compact = 0)
	Local total
	Local i
	For L_MsgColors.L_MsgColor = Each L_MsgColor
		If fntid = L_MsgColors\ID Then
			Exit
		End If
	Next
	For i = 1 To Len(txt$)
		total = total + ImageWidth(L_MsgColors\imgChars[Asc(Mid(txt$, i, 1))]) - 2 - compact
	Next
	total = total + 2 + compact
	Return total
End Function
; ---------------------------
Function L_Text(fntid, x, y, txt$, compact = 0)
	Local i
	Local off
	For L_MsgColors.L_MsgColor = Each L_MsgColor
		If fntid = L_MsgColors\ID Then
			Exit
		End If
	Next	
	For i = 1 To Len(txt$)
		DrawImage L_MsgColors\imgChars[Asc(Mid(txt$, i, 1))], x + off, y
		off = off + ImageWidth(L_MsgColors\imgChars[i])-2-compact
	Next
End Function
; ---------------------------
Function L_OutlineText(x, y, txt$, center_x, center_y)
	; Local Vars
	Local R
	Local G
	Local B
	r = ColorRed()
	g = ColorGreen()
	b = ColorBlue()
	Color 0, 0, 0
	Text(x+1, y, txt$, center_x, center_y)
	Text(x-1, y, txt$, center_x, center_y)
	Text(x, y+1, txt$, center_x, center_y)
	Text(x, y-1, txt$, center_x, center_y)
	Color r, g, b
	Text(x, y, txt$, center_x, center_y)
End Function
; ---------------------------
Function L_isVowel(char$)
	char$ = Upper(Left(char$, 1))
	If char$ = "A" Or char$ = "E" Or char$ = "I" Or char$ = "O" Or char$ = "U" Or char$ = "Y" Then
		Return True
	Else
		Return False
	End If
End Function
; ---------------------------
Function L_Zerotize$(Number, Length)
	; Local Vars
	Local Final$
	Final$ = Number
	While Len(Final) < Length
		Final = "0" + Final
	Wend
	Return Final
End Function