; This source code is copyrighted (c) to Jocelyn Perreault.
; It may not be used to create another game. You can modify
; it (i.e., make mods) but you need to notify me if you do
; so, and my name must remain in the credits. It cannot be
; redistributed (the source code and the game itself).
; Thanks,
;                 Jocelyn "GoSsE Korupted" Perreault
;                  gosse@nuloen.com
;                  Nuclear Loaded Entertainment
;                  http://www.nuloen.com/

; FEED SOME CONSTANTS
zoneDepth = 30         ; ZONE BORDER STEEP
maxTerrains = 5        ; NUMBER OF TERRAIN HEIGHT MAPS
maxGroups = 36		   ; MAXIMUM NUMBER OF GROUPS
maxRaceGroup = 4       ; MAXIMUM NUMBER OF GROUPS FOR THE SAME RACE
AttackRange = 85       ; UNITS TO ENABLE ATTACK/TARGET
AggroRange = 400       ; RANGE AT WHICH ENEMIES AGGRO
RestSpeed = 10		   ; TIME SPEED MULTIPLIER WHEN YOU REST
DStatMin = 10          ; MINIMUM VALUE FOR A DERIVED STAT
camSpeed# = 2          ; THE SPEED AT WHICH THE CAMERA PLACES ITSELF
MoveSpeed# = 3.5       ; THE SPEED AT WHICH MODELS MOVE FORWARD
AnimSpeed# = 0.17      ; ANIMATION SPEED
TargetRange# = 20	   ; THE RANGE AT WHICH YOU CAN SELECT AN NPC AROUND
CHAOSZONEOFFSET = 50   ; ZONE OFFSET FOR CHAOS (50)

Global room
Global lowNPC
Global highNPC
Global props.Prop
Global BloodEmitters.BloodEmitter
Global zx
Global zy
Global camDefault
Global HEAT
Global MOVT
Global MORT
Global ST
Global DE
Global ME
Global DAM
Global AB
Global maxItems
Global GOL

; LOAD BASIC SOUNDS
sndClick = LoadSound("sounds\click.wav")
sndInvalid = LoadSound("sounds\invalid click.wav")

; SEED
SeedRnd MilliSecs()

; TYPES DEFINITIONS
Type NPCRace
	Field Race$         ; NAME OF THE RACE
	Field sndAtk		; SOUND HANDLE
	Field sndHit		; SOUND HANDLE
	Field sndDie		; SOUND HANDLE
End Type

Type Group
	Field Name$		    ; NAME OF THE GROUP
	Field RaceID        ; RACE OF THE GROUP
	Field ALI           ; ALIGNMENT OF THE GROUP
	Field Faction		; FACTION TOWARD THIS GROUP
	Field ModelNumber$  ; MODEL NUMBER OF THIS FACTION
	Field Stats#        ; STATS MULTIPLIER
End Type

Type Prop
	Field ModelPath$    ; PATH TO MODEL
	Field TexPath$      ; PATH TO TEXTURE
	Field Model         ; TEMP MODEL HANDLE
	Field ColModel      ; TEMP COLLISION MODEL
	Field X				; X POSITION
	Field Y				; Y POSITION
	Field H				; HEIGHT POSITION
	Field Angle			; YAW ANGLE
	Field WS#		    ; WIDTH SCALE
	Field HS#		    ; HEIGHT SCALE
	Field Pitch			; PITCH ANGLE
	Field Radius		; RADIUS
	Field ZX			; ZONE ID X
	Field ZY			; ZONE ID Y
	Field Treasure      ; TREASURE OR NOT?
	Field GOL           ; GOLDS IN IT
	Field Shop          ; SHOP ID, IF IT'S A SHOP
	Field Quest			; QUEST ID, IF IT'S A QUEST
End Type

Type NPC
	Field Number$		; NPC MODEL NUMBER
	Field TexNumber$	; ITS TEXTURE NUMBER
	Field Model			; TEMP MODEL HANDLE
	Field WpnModel		; TEMP WEAPON MODEL HANDLE
	Field X				; X POSITION
	Field Y				; Y POSITION
	Field ZX			; ZONE ID X
	Field ZY			; ZONE ID Y
	Field H#			; HEIGHT SCALE
	Field W#			; WEIGHT SCALE
	Field moving		; CURRENTLY MOVING? TRUE/FALSE
	Field turning		; CURRENTLY TURNING? (AMOUNT -2/0/2)
	Field zoning		; CURRENTLY ZONING? TRUE/FALSE
	Field name$			; THE NPC NAME
	Field Race$			; RACE OF THE NPC
	Field group			; GROUP NUMBER IT IS PART
	Field lastFaction	; LAST FACTION WHEN PLAYER WAS CROSSED
	Field HEA			; HEALTH LEVEL
	Field HEAT			; HEALTH TOTAL
	Field MOV#			; MOVE LEVEL
	Field MOVT#			; MOVE TOTAL
	Field MOR			; MORALE LEVEL
	Field MORT			; MORALE TOTAL
	Field ST			; STRENGTH SKILL
	Field DE			; DEXTERITY SKILL
	Field ME			; MENTAL SKILL
	Field DAM			; DAMAGES
	Field AB			; ABSORPTION
	Field ALI			; ALIGNMENT
	Field REA			; REACTION
	Field GOL			; GOLDS CARRIED
	Field Dead			; DEAD OR NOT
	Field Items$        ; ALL ITEMS CARRIED
	Field Step1         ; FIRST FOOT STEP
	Field Step2			; SECOND FOOT STEP
	Field Alpha#		; ALPHA CHANGE WHEN DYING
	Field Scale#		; SCALE CHANGE WHEN DYING
End Type

Type Zone
	Field exit1$
	Field exit2$
	Field exit3$
	Field exit4$
	Field exit5$
	Field terrain
	Field texture
End Type

Type Valuable
	Field ID$			; NUMBER OF VALUABLE
	Field Value$		; VALUE OF VALUABLE
End Type

Type LogData
	Field Name$			; LOG NAME
	Field Value$        ; LOG VALUE
End Type

Type Shop
	Field Items$		; ITEMS IT HAS (LIKE NPCS)
	Field GOL			; HOW MANY GOLDS THE SHOP KEEPER HAS
	Field POT			; HOW MANY POTIONS
	Field Weapons$		; WEAPONS FOR SALE
	Field Armors$		; ARMORS FOR SALE
End Type

Type Terrain
	Field ID			; ID OF THE TEXTURE
	Field Life#			; LIFE PERCENTAGE
End Type

Type BloodEmitter
	Field emt			; EMITTER HANDLE
End Type

Type Quest
	Field ID			; ID OF THE QUEST
	Field Typ			; TYPE OF THE QUEST (RETRIEVE, KILL, CONVINCE)
	Field Name$			; NAME OF THE OBJECTIVE
	Field Done			; IS THE QUEST DONE?
End Type

; VARIABLES
Dim zones.Zone(100, 100)
Dim groups.Group(maxGroups)
Dim checkZones(4)
Dim Exits$(5)
Dim pvtExit(5)
Dim enabledDir(4)
Dim CountDir(4)
Dim OtherDir(4)
Dim GroupRaces$(100)
Dim Prefixes$(100)
Dim Suffixes$(100)
Dim FirstNames$(50)
Dim LastNames$(50)
Dim Valuables$(500)
Dim YItems(10)
Dim NItems(10)
Dim imgShots(10)
Dim SlotName$(10)
Dim VNs$(10)
Dim Empty(10)
Dim Info$(10, 11)
MOV# = 0
logPath$ = "saves\mixed"

Global SkipIntro

; LOAD TERRAINS
filTerrains = ReadFile("data\terrains.dat")
	Repeat
		Terr$ = ReadLine(filTerrains)
		Terrains.Terrain = New Terrain
		Terrains\ID = Left(Terr, 2)
		Terrains\Life = Mid(Terr, 3)
		maxTerTex = maxTerTex + 1
	Until Eof(filTerrains)
CloseFile filTerrains

; LOAD RACES
count = 0
filRaces = ReadFile("data\races.dat")
	Repeat
		GroupRaces(count) = ReadLine(filRaces)
		Count = Count + 1
	Until Eof(filRaces)
CloseFile filRaces
MaxRaces = count - 1

; LOAD PREFIXES
count = 0
filPrefixes = ReadFile("data\prefixes.dat")
	Repeat
		Prefixes(count) = ReadLine(filPrefixes)
		Count = Count + 1
	Until Eof(filPrefixes)
CloseFile filPrefixes
MaxPrefixes = count - 1

; LOAD SUFFIXES
count = 0
filSuffixes = ReadFile("data\suffixes.dat")
	Repeat
		Suffixes(count) = ReadLine(filSuffixes)
		Count = Count + 1
	Until Eof(filSuffixes)
CloseFile filSuffixes
MaxSuffixes = count - 1

; LOAD ITEMS
count = 0
filItems = ReadFile("data\valuables.dat")
	Repeat
		Valuables(count) = ReadLine(filItems)
		Count = Count + 1
	Until Eof(filItems)
CloseFile filItems
MaxItems = count - 1

; PARTICLES
; Create the quad mesh for particles
Global objQuad = PW_CreateQuad()
HideEntity objQuad
; Particles textures
Global texBlood = LoadTexture("textures\blood.tga", 2)

Function getLow(val$)
	Return Mid(val, 1, 3)
End Function

Function getHigh(val$)
	Return Mid(val, 4, 3)
End Function

Function getWeight(val$)
	Return Mid(val, 7, 2)
End Function

Function getFragility(val$)
	Return Mid(val, 9, 2)
End Function

Function getName$(val$)
	Return Mid(val, 11)
End Function

Function getLog$(lg$)
	found = False
	For logs.LogData = Each LogData
		If logs\name = lg Then
			found = True
			Return logs\value
		End If
	Next
	If Not found Then
		Return ""
	End If
End Function

Function setLog(lg$, val$)
	found = False
	For logs.LogData = Each LogData
		If logs\name = lg Then
			found = True
			logs\value = val
		End If
	Next
	If Not found Then
		logs.LogData = New LogData
		logs\name = lg
		logs\value = val
	End If
End Function

Function incLog(lg$)
	SetLog(lg, Int(GetLog(lg))+1)
End Function

Function addLog(lg$, inc)
	SetLog(lg, Int(GetLog(lg)) + Int(inc))
End Function

Function loading()
	DrawImage LoadImage("images\loading.jpg"), 170, 190
	Flip

End Function

Function getWEA$(WTY, WME)
	WpnName$ = ""
	Select WTY
		Case 2
			WpnName = " Axe"
		Case 3
			WpnName = " Mace"
		Case 7
			WpnName = " Longsword"
		Case 11
			WpnName = " Battleaxe"
		Case 14
			WpnName = " Broadsword"
		Case 20
			WpnName = " Flail"
	End Select
	Select WME
		Case 1
			WpnName = "Iron" + WpnName
		Case 4
			WpnName = "Steel" + WpnName
		Case 8
			WpnName = "Mythril" + WpnName
		Case 13
			WpnName = "Dwarven" + WpnName
		Case 19
			WpnName = "Emerald" + WpnName
		Case 24
			WpnName = "Adamantium" + WpnName
		Case 30
			WpnName = "Daemonic" + WpnName
	End Select
	Return WpnName
		
End Function

Function getARM$(AME)
	ArmName$ = " Plate"
	Select AME
		Case 1
			ArmName = "Iron" + ArmName
		Case 4
			ArmName = "Steel" + ArmName
		Case 8
			ArmName = "Mythril" + ArmName
		Case 13
			ArmName = "Dwarven" + ArmName
		Case 19
			ArmName = "Emerald" + ArmName
		Case 24
			ArmName = "Adamantium" + ArmName
		Case 30
			ArmName = "Daemonic" + ArmName
	End Select
	Return ArmName

End Function